/******************************************************************************
    QtAV:  Multimedia framework based on Qt and FFmpeg
    Copyright (C) 2012-2016 Wang Bin <wbsecg1@gmail.com>

*   This file is part of QtAV (from 2015)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

#ifndef QTAVWIDGETS_H
#define QTAVWIDGETS_H

#include <QtAVWidgets/version.h>
#include <QtAVWidgets/global.h>
#include <QtAVWidgets/GraphicsItemRenderer.h>
#include <QtAVWidgets/WidgetRenderer.h>
//#include <QtAVWidgets/GLWidgetRenderer.h>
#include <QtAVWidgets/GLWidgetRenderer2.h>
#include <QtAVWidgets/VideoPreviewWidget.h>

#endif // QTAVWIDGETS_H
